

######################################################################################################################################
######################################################################################################################################
### Estimating extinction rate using an unbounded subplex routine
######################################################################################################################################
######################################################################################################################################

library(diversitree)

bd.b.vect <- c()
bd.e.vect <- c()
bd.loglik <- c()
t.vect <- c()
yule.b.vect <- c()
yule.loglik <- c()
n.taxa.vect<-c()

tree<-read.tree("e00s00.trees")
for(i in 1:2000){
	bd <- NA
	yule <- NA
	try(p <- starting.point.bd(tree[[i]]), silent=TRUE)
	try(lik.bd <- make.bd(tree[[i]]), silent=TRUE)
	try(bd <- find.mle(lik.bd,p,method="subplex"), silent=TRUE)
	try(lik.yule <- constrain(lik.bd,mu~0), silent=TRUE)
	try(yule <- find.mle(lik.yule,p,method="subplex"), silent=TRUE)
	if(!is.na(bd[1]) & !is.na(yule[1])) {
		t.vect <- append(t.vect, max(branching.times(tree[[i]])))
		bd.b.vect <- append(bd.b.vect,bd$par[1])
		bd.e.vect <- append(bd.e.vect,bd$par[2])
		bd.loglik <- append(bd.loglik,bd$lnLik)
		yule.b.vect <- append(yule.b.vect,yule$par[1])
		yule.loglik <- append(yule.loglik,yule$lnLik)
		n.taxa.vect <- append(n.taxa.vect,Ntip(tree[[i]]))
	}
}
save(t.vect,bd.b.vect,bd.e.vect,bd.loglik, yule.b.vect,yule.loglik, n.taxa.vect, file="e00s00.Rsave",compress=TRUE)


bd.b.vect <- c()
bd.e.vect <- c()
bd.loglik <- c()
t.vect <- c()
yule.b.vect <- c()
yule.loglik <- c()
n.taxa.vect<-c()

tree<-read.tree("e00s06sT.trees")
for(i in 1:2000){
	bd <- NA
	yule <- NA
	try(p <- starting.point.bd(tree[[i]]), silent=TRUE)
	try(lik.bd <- make.bd(tree[[i]]), silent=TRUE)
	try(bd <- find.mle(lik.bd,p,method="subplex"), silent=TRUE)
	try(lik.yule <- constrain(lik.bd,mu~0), silent=TRUE)
	try(yule <- find.mle(lik.yule,p,method="subplex"), silent=TRUE)
	if(!is.na(bd[1]) & !is.na(yule[1])) {
		t.vect <- append(t.vect, max(branching.times(tree[[i]])))
		bd.b.vect <- append(bd.b.vect,bd$par[1])
		bd.e.vect <- append(bd.e.vect,bd$par[2])
		bd.loglik <- append(bd.loglik,bd$lnLik)
		yule.b.vect <- append(yule.b.vect,yule$par[1])
		yule.loglik <- append(yule.loglik,yule$lnLik)
		n.taxa.vect <- append(n.taxa.vect,Ntip(tree[[i]]))
	}
}
save(t.vect,bd.b.vect,bd.e.vect,bd.loglik, yule.b.vect,yule.loglik, n.taxa.vect, file="e00s06sT.Rsave",compress=TRUE)



bd.b.vect <- c()
bd.e.vect <- c()
bd.loglik <- c()
t.vect <- c()
yule.b.vect <- c()
yule.loglik <- c()
n.taxa.vect<-c()

tree<-read.tree("e00s06T.trees")
for(i in 1:2000){
	bd <- NA
	yule <- NA
	try(p <- starting.point.bd(tree[[i]]), silent=TRUE)
	try(lik.bd <- make.bd(tree[[i]]), silent=TRUE)
	try(bd <- find.mle(lik.bd,p,method="subplex"), silent=TRUE)
	try(lik.yule <- constrain(lik.bd,mu~0), silent=TRUE)
	try(yule <- find.mle(lik.yule,p,method="subplex"), silent=TRUE)
	if(!is.na(bd[1]) & !is.na(yule[1])) {
		t.vect <- append(t.vect, max(branching.times(tree[[i]])))
		bd.b.vect <- append(bd.b.vect,bd$par[1])
		bd.e.vect <- append(bd.e.vect,bd$par[2])
		bd.loglik <- append(bd.loglik,bd$lnLik)
		yule.b.vect <- append(yule.b.vect,yule$par[1])
		yule.loglik <- append(yule.loglik,yule$lnLik)
		n.taxa.vect <- append(n.taxa.vect,Ntip(tree[[i]]))
	}
}
save(t.vect,bd.b.vect,bd.e.vect,bd.loglik, yule.b.vect,yule.loglik, n.taxa.vect, file="e00s06T.Rsave",compress=TRUE)



bd.b.vect <- c()
bd.e.vect <- c()
bd.loglik <- c()
t.vect <- c()
yule.b.vect <- c()
yule.loglik <- c()
n.taxa.vect<-c()




######################################################################################################################################
######################################################################################################################################
### Estimating exinction rates using a bounded Nelder-Mead routine
######################################################################################################################################
######################################################################################################################################

library(diversitree)
library(nloptr)

bisse.bound<-function(phy){
	
	obj <- NULL 
	dev.optimize <- function(p, phy){

		#backsolve for b and d
		lambda.p <- p[1]/(1+p[2]) 
		mu.p <- (p[2]*p[1])/(1+p[2])
		
		lik <- make.bd(phy)
		loglik <- lik(c(lambda.p,mu.p))
		
		-loglik
	}
	
	lower <- c(0,0)
	upper <- c(100,1)
	
	ip <- c(.5, .5)
	
	opts <- list("algorithm"="NLOPT_LN_NELDERMEAD", "maxeval"="1000000", "ftol_rel"=.Machine$double.eps^0.5)
	out = nloptr(x0=ip, eval_f=dev.optimize, lb=lower, ub=upper, opts=opts, phy=phy)
	
	obj$loglik <- -out$objective
	obj$param.est <- out$solution
	
	obj
}



bd.t.vect <- c()
bd.e.vect <- c()
bd.loglik <- c()
t.vect <- c()
n.taxa.vect<-c()

tree<-read.tree("e00s00.trees")
for(i in 1:2000){
	print(Ntip(tree[[i]]))
	try(bd<-bisse.bound(tree[[i]]))
	if(!is.na(bd[1])) {
		t.vect <- c(append(t.vect, max(branching.times(tree[[i]]))))
		bd.loglik<-append(bd.loglik, bd$loglik)
		bd.t.vect<-append(bd.t.vect,bd$param.est[1])
		bd.e.vect<-append(bd.e.vect,bd$param.est[2])
		n.taxa.vect<-append(n.taxa.vect,Ntip(tree[[i]]))
	}
	print("done!")
}
save(t.vect, bd.loglik, bd.t.vect,bd.e.vect,n.taxa.vect,file="e00s00_NM.Rsave",compress=TRUE)



bd.t.vect <- c()
bd.e.vect <- c()
bd.loglik <- c()
t.vect <- c()
n.taxa.vect<-c()

tree<-read.tree("e00s06T.trees")
for(i in 1:2000){
	try(bd<-bisse.bound(tree[[i]]))
	if(!is.na(bd[1])) {
		print(Ntip(tree[[i]]))
		t.vect <- c(append(t.vect, max(branching.times(tree[[i]]))))
		bd.loglik<-append(bd.loglik, bd$loglik)
		bd.t.vect<-append(bd.t.vect,bd$param.est[1])
		bd.e.vect<-append(bd.e.vect,bd$param.est[2])
		n.taxa.vect<-append(n.taxa.vect,Ntip(tree[[i]]))
		print("done!")
	}
}
save(t.vect, bd.loglik, bd.t.vect,bd.e.vect,n.taxa.vect,file="e00s06T_NM.Rsave",compress=TRUE)



bd.t.vect <- c()
bd.e.vect <- c()
bd.loglik <- c()
t.vect <- c()
n.taxa.vect<-c()

tree<-read.tree("e00s06sT.trees")
for(i in 1:2000){
	try(bd<-bisse.bound(tree[[i]]))
	if(!is.na(bd[1])) {
		t.vect <- c(append(t.vect, max(branching.times(tree[[i]]))))
		bd.loglik<-append(bd.loglik, bd$loglik)
		bd.t.vect<-append(bd.t.vect,bd$param.est[1])
		bd.e.vect<-append(bd.e.vect,bd$param.est[2])
		n.taxa.vect<-append(n.taxa.vect,Ntip(tree[[i]]))
	}
}
save(t.vect, bd.loglik, bd.t.vect,bd.e.vect,n.taxa.vect,file="e00s06sT_NM.Rsave",compress=TRUE)



